/**
   @version 1.0 2004-06-10
   @author Gonzalo Ojeda Figueroa
*/

import java.io.*;
import java.net.*;
//import java.util.zip.*;
import java.lang.*;

/**
   Este programa implementa un servidor que maneja multiples threads y
   escucha en el puerto 8189 en este caso, recibe archivos por ese puerto
   los copa al mismo directorio de ejecucion y guarda los datos del archivo
   (su peso, nombre y fecha de la recepcion) en un archivo llamado entregadas.txt
   Basado en el programa ThreadedEchoServer de Cay Horstmann.
*/
public class ThreadedFileServer
{
   /**
      En el main del programa, se define el puerto sobre el cual el servidor
      escuchara, y se crea un ServerSocket que estara escuchando ese puerto,
      luego se entra a un ciclo que empieza a recibir conecciones y para cada
      una de ellas crea un thread que la maneje.
   */
   public static void main(String[] args )
   {
      try
      {
         final int PUERTO = 8189;   // puerto que escuchara el server
         int i = 1;
         ServerSocket s = new ServerSocket(PUERTO);   // ServerSocker que escucha el puerto

         for (;;)     // espera eternamente la llegada de conexiones
         {
            Socket incoming = s.accept( );   // acepta la conexion al realizarse
            // crea un nuevo Thread para atender la coneccion
            Thread t = new ThreadedServerHandler(incoming, i);
            t.start();     // inicializa este nuevo Thread
            i++;
         }
      }
      catch (Exception e)
      {
         e.printStackTrace();       // captura posibles exepciones
      }
   }
}

/**
   Esta clase maneja la coneccion de los clientes por el socket
   escuchado por el servidor, con un Thread para cada coneccion.
*/

class ThreadedServerHandler extends Thread
{
   /**
      Construye un manejador para tener cada coneccion en un Thread separado
      @param i el socket en el cual se establecio la coneccion
      @param c contador, para llevar una cuenta de cuantos clientes se han
               conectado
   */
   public ThreadedServerHandler(Socket i, int c)
   {
      incoming = i; counter = c;
   }

   /**
         Funcion con los Procesos a ejecutar para cada una de las conecciones.
   */
   public void run()
   {
      // lazo que captura posibles excepciones en la recepcion del archivo,
      // manejo del mismo, o con la coneccion.
      try
      {
         DataInputStream inbound = new DataInputStream(
            incoming.getInputStream() );  // para recibir los datos desde el socket

         long largo;    // antes de enviar los archivos se envia el largo de los mismos
         largo = inbound.readLong();   // ser recibe el largo

//-------------------

        // crear un archivo temporal para recibir el archivo
        File inputFile = new File ("temp"+ counter + ".zip");
        FileOutputStream out = new FileOutputStream(inputFile);
        for(int i=0; i<largo; i++)  // se recibe el archivo byte a byte
           out.write(inbound.read());  // se lo escribe al archivo

//-------------------

         String name;   // para almacenar el nombre original del archivo
         BufferedReader d  // para poder usar readLine, se necesita un BufferedReader
          = new BufferedReader(new InputStreamReader(incoming.getInputStream()));
          name = d.readLine();   // lee el nombre del archivo

//--------------------------------------

         // un nuevo archivo con el nombre del archivo original
         File fout = new File (name);
         // se ve si archivo existe, no se lo sobreescribe, sino que se lo renombra
         // agregando un numero correlativo al final de su nombre
         if (fout.exists())
         {
            int count = 1;
            while (fout.exists())
            {
               String tempName = name + "." + count;
               fout = new File (tempName);
               count++;
            }
         }
         // se renombra el archivo al nombre original que traia o con numero
         // correlativo en caso de ya existir el nombre original.
         inputFile.renameTo(fout);

//---------------------------------------

         inbound.close();  // cerrar DataInputStram
         incoming.close(); // cerrar Socket

         FileWrite wrt = new FileWrite(); // objeto para escribir registro de recibidas
         // se llama al metodo que escribe el registro
         wrt.writeString(name, fout.length());
      }
      catch (Exception e)  // capturar excepciones
      {
         e.printStackTrace();
      }
   }

   private Socket incoming;   // socket en en cual se realiza la coneccion
   private int counter;       // contador para llevar cuenta de clientes
}

